# Copyright (c) 2012 Red Hat, Inc. All rights reserved. This copyrighted material 
# is made available to anyone wishing to use, modify, copy, or 
# redistribute it subject to the terms and conditions of the GNU General 
# Public License v.2.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# Author: Greg Nichols
#

import dbus

from rhcert.tags import Constants

from device import Device

class DBusDevice(Device):
    def __init__(self, dbusClass, dbusObject):
        Device.__init__(self)
        self.dbusClass = dbusClass
        self.dbusObject = dbusObject
        self.dbusInterface = dbus.Interface(self.dbusObject, dbus.PROPERTIES_IFACE)
        self.getProperties()

    def dump(self):
        print "\nUDI: " + self.getUDI() + "\n----------------------------------------"
        # print self.dbusObject.Introspect()
        for (name, value) in self.getProperties().iteritems():
            print "%s: %s" % (name, value)

    def getProperty(self, name):
        try:
            return self.dbusObject.Get(self.dbusClass, name)
        except:
            return ""

    def getProperties(self):
        # refresh them
        self.properties = dict()
        for (property, value) in self.dbusObject.GetAll(self.dbusClass).iteritems():
            # skip binary blobs alltogether
            if "Blob" in property:
                continue
            # convert list values to strings with comma separation
            try:
                if isinstance(value, dbus.Array):
                    value = ", ".join(value)
            except TypeError:
                pass
            self.properties[property] = value
        return self.properties


class UDisksDevice(DBusDevice):

    def __init__(self, object):
        DBusDevice.__init__(self, dbusClass="org.freedesktop.UDisks.Device", dbusObject=object)
        self.setUDI(self.getProperty("NativePath"))

    def dump(self):
        DBusDevice.dump(self)
        for attribute in ["DeviceFile", "NativePath", "IdLabel", "IdUsage", "IdUuid", "DeviceSize"]:
            print attribute + ": " + str(self.getProperty(attribute))

    def getMountPaths(self):
        paths = list()
        if self.getProperty("DeviceIsMounted"):
            paths = self.getProperty("DeviceMountPaths")
        return paths

    def getSource(self):
        return Constants.dbus


class UDisksDrive(UDisksDevice):

    def __init__(self, object):
        UDisksDevice.__init__(self, object)

    def dump(self):
        UDisksDevice.dump(self)

    def getProduct(self):
        return self.getProperty("DriveModel")

class UDisksPartition(UDisksDevice):

    def __init__(self, object):
        UDisksDevice.__init__(self, object)

class UDisksAdapter(DBusDevice):

    def __init__(self, object):
        DBusDevice.__init__(self, dbusClass="org.freedesktop.UDisks.Adapter", dbusObject=object)
        self.setUDI(self.getProperty("NativePath"))

    def getSource(self):
        return Constants.udisks

    def getProduct(self):
        return self.getProperty("Model")

